@@ -35,7 +35,7 @@ impl<const N: usize> minicbor::Encode for SkipCbor<N> {
 pub type SomeSkipCbor = SkipCbor<0>;
 
 #[derive(Encode, Decode, Debug, PartialEq, Clone)]
-pub struct VrfCert(#[n(0)] ByteVec, #[n(1)] ByteVec);
+pub struct VrfCert(#[n(0)] pub ByteVec, #[n(1)] pub ByteVec);
 
 #[derive(Encode, Decode, Debug, PartialEq, Clone)]
 pub struct HeaderBody {
@@ -179,11 +179,103 @@ pub type Hash28 = ByteVec;
 pub type Hash32 = ByteVec;
 
 pub type PoolKeyhash = Hash28;
-pub type Epoch = i64;
+pub type Epoch = u64;
 pub type Genesishash = SkipCbor<5>;
 pub type GenesisDelegateHash = SkipCbor<6>;
 pub type VrfKeyhash = Hash32;
-pub type MoveInstantaneousReward = SkipCbor<8>;
+
+/* move_instantaneous_reward = [ 0 / 1, { * stake_credential => delta_coin } / coin ]
+; The first field determines where the funds are drawn from.
+; 0 denotes the reserves, 1 denotes the treasury.
+; If the second field is a map, funds are moved to stake credentials,
+; otherwise the funds are given to the other accounting pot.
+ */
+
+#[derive(Debug, PartialEq, PartialOrd)]
+pub enum InstantaneousRewardSource {
+    Reserves,
+    Treasury,
+}
+
+impl<'b> minicbor::decode::Decode<'b> for InstantaneousRewardSource {
+    fn decode(d: &mut minicbor::Decoder<'b>) -> Result<Self, minicbor::decode::Error> {
+        let variant = d.u32()?;
+
+        match variant {
+            0 => Ok(Self::Reserves),
+            1 => Ok(Self::Treasury),
+            _ => Err(minicbor::decode::Error::Message("invalid funds variant")),
+        }
+    }
+}
+
+impl minicbor::encode::Encode for InstantaneousRewardSource {
+    fn encode<W: minicbor::encode::Write>(
+        &self,
+        e: &mut minicbor::Encoder<W>,
+    ) -> Result<(), minicbor::encode::Error<W::Error>> {
+        let variant = match self {
+            Self::Reserves => 0,
+            Self::Treasury => 1,
+        };
+
+        e.u32(variant)?;
+
+        Ok(())
+    }
+}
+
+#[derive(Debug, PartialEq, PartialOrd)]
+pub enum InstantaneousRewardTarget {
+    StakeCredentials(BTreeMap<StakeCredential, i64>),
+    OtherAccountingPot(Coin),
+}
+
+impl<'b> minicbor::decode::Decode<'b> for InstantaneousRewardTarget {
+    fn decode(d: &mut minicbor::Decoder<'b>) -> Result<Self, minicbor::decode::Error> {
+        let datatype = d.datatype()?;
+
+        match datatype {
+            minicbor::data::Type::Map => {
+                let a = d.decode()?;
+                Ok(Self::StakeCredentials(a))
+            }
+            _ => {
+                let a = d.decode()?;
+                Ok(Self::OtherAccountingPot(a))
+            }
+        }
+    }
+}
+
+impl minicbor::encode::Encode for InstantaneousRewardTarget {
+    fn encode<W: minicbor::encode::Write>(
+        &self,
+        e: &mut minicbor::Encoder<W>,
+    ) -> Result<(), minicbor::encode::Error<W::Error>> {
+        match self {
+            InstantaneousRewardTarget::StakeCredentials(a) => {
+                a.encode(e)?;
+                Ok(())
+            }
+            InstantaneousRewardTarget::OtherAccountingPot(a) => {
+                a.encode(e)?;
+                Ok(())
+            }
+        }
+    }
+}
+
+#[derive(Encode, Decode, Debug, PartialEq, PartialOrd)]
+#[cbor]
+pub struct MoveInstantaneousReward {
+    #[n(0)]
+    pub source: InstantaneousRewardSource,
+
+    #[n(1)]
+    pub target: InstantaneousRewardTarget,
+}
+
 pub type Margin = SkipCbor<9>;
 pub type RewardAccount = ByteVec;
 pub type PoolOwners = SkipCbor<11>;
@@ -255,10 +347,10 @@ pub type PoolMetadataHash = Hash32;
 #[derive(Encode, Decode, Debug, PartialEq)]
 pub struct PoolMetadata {
     #[n(0)]
-    url: String,
+    pub url: String,
 
     #[n(1)]
-    hash: PoolMetadataHash,
+    pub hash: PoolMetadataHash,
 }
 
 pub type AddrKeyhash = Hash28;
@@ -266,8 +358,8 @@ pub type Scripthash = Hash28;
 
 #[derive(Debug, PartialEq)]
 pub struct RationalNumber {
-    numerator: i64,
-    denominator: u64,
+    pub numerator: i64,
+    pub denominator: u64,
 }
 
 impl<'b> minicbor::decode::Decode<'b> for RationalNumber {
@@ -299,7 +391,7 @@ impl minicbor::encode::Encode for RationalNumber {
 
 pub type UnitInterval = RationalNumber;
 
-#[derive(Debug, PartialEq)]
+#[derive(Debug, PartialEq, PartialOrd, Eq, Ord)]
 pub enum StakeCredential {
     AddrKeyhash(AddrKeyhash),
     Scripthash(Scripthash),
@@ -806,7 +898,7 @@ impl minicbor::encode::Encode for PlutusData {
 
 /// A struct that forces encode / decode using indef arrays
 #[derive(Debug, PartialEq, Eq, PartialOrd, Ord)]
-pub struct IndefVec<A>(Vec<A>);
+pub struct IndefVec<A>(pub Vec<A>);
 
 impl<'b, A> minicbor::decode::Decode<'b> for IndefVec<A>
 where
@@ -913,9 +1005,9 @@ where
 #[derive(Encode, Decode, Debug, PartialEq)]
 pub struct ExUnits {
     #[n(0)]
-    mem: u32,
+    pub mem: u32,
     #[n(1)]
-    steps: u32,
+    pub steps: u32,
 }
 
 #[derive(Encode, Decode, Debug, PartialEq)]
@@ -1090,7 +1182,7 @@ impl minicbor::Encode for Metadatum {
 /// as the underlaying struct for storage of the items (as opposed to a BTreeMap
 /// or HashMap).
 #[derive(Debug, PartialEq, Eq, PartialOrd, Ord)]
-pub struct Metadata(Vec<(Metadatum, Metadatum)>);
+pub struct Metadata(pub Vec<(Metadatum, Metadatum)>);
 
 impl Deref for Metadata {
     type Target = Vec<(Metadatum, Metadatum)>;
